package com.diagbot.facade;

import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.entity.node.Indication;
import com.diagbot.entity.node.IndicationCondition;
import com.diagbot.repository.IndicationRepository;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.ConditionIndexVO;
import com.diagbot.vo.IndicationPageVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/11/17 13:53
 */
@Component
public class IndicationManFacade {
    @Autowired
    IndicationRepository indicationRepository;
    @Autowired
    DictionaryFacade dictionaryFacade;

    /**
     * 获取开单合理项列表
     *
     * @param indicationPageVO
     * @return
     */
    public Page<Indication> getPage(IndicationPageVO indicationPageVO) {
        List<DictionaryInfoDTO> dicTypeConvert = dictionaryFacade.getListByGroupType(8);
        List<DictionaryInfoDTO> dicIndicationManType = dictionaryFacade.getListByGroupType(11);
        Map<String, String> dicTypeConvertMap
                = EntityUtil.makeMapWithKeyValue(dicTypeConvert, "name", "val");
        Map<String, String> dicIndicationManTypeMap
                = EntityUtil.makeMapWithKeyValue(dicIndicationManType, "val", "name");
        List<String> labels = Lists.newArrayList();
        if (ListUtil.isNotEmpty(indicationPageVO.getTypes())) {
            for (Integer type : indicationPageVO.getTypes()) {
                if (dicIndicationManTypeMap.containsKey(type.toString())
                        && dicTypeConvertMap.containsKey(dicIndicationManTypeMap.containsKey(type.toString()))) {
                    labels.add(dicTypeConvertMap.get(dicIndicationManTypeMap.get(type.toString())));
                }
            }
        }

        Pageable pageable = PageRequest.of(indicationPageVO.getNumber(), indicationPageVO.getSize());
        Page<Indication> page = indicationRepository.getPage(indicationPageVO.getLabels(),
                indicationPageVO.getConceptName(),
                indicationPageVO.getConditionName(),
                indicationPageVO.getRelationStatus(),
                pageable);
        return page;
    }

    /**
     * 条件明细检索
     *
     * @param conditionIndexVO
     * @return
     */
    public List<IndicationCondition> conditionIndex(ConditionIndexVO conditionIndexVO) {
        //TODO 输血单独处理


        List<DictionaryInfoDTO> dicTypeConvert = dictionaryFacade.getListByGroupType(8);
        List<DictionaryInfoDTO> dicIndicationManType = dictionaryFacade.getListByGroupType(11);
        Map<String, String> dicTypeConvertMap
                = EntityUtil.makeMapWithKeyValue(dicTypeConvert, "name", "val");
        Map<String, String> dicIndicationManTypeMap
                = EntityUtil.makeMapWithKeyValue(dicIndicationManType, "val", "name");
        if (dicIndicationManTypeMap.containsKey(conditionIndexVO.getType().toString())
                && dicTypeConvertMap.containsKey(dicIndicationManTypeMap.containsKey(conditionIndexVO.getType().toString()))) {
            conditionIndexVO.setConceptLabel(dicTypeConvertMap.get(dicIndicationManTypeMap.get(conditionIndexVO.getType().toString())));
        }
        List<String> relationTypes = indicationRepository.getRelationTypes(conditionIndexVO.getConceptLabel());
        List<String> conditionLabels = Lists.newArrayList();
        //条件明细筛选范围
        if (ListUtil.isNotEmpty(relationTypes)) {
            for (String relationType : relationTypes) {
                String[] splitArr = relationType.split("禁忌");
                if (splitArr != null && splitArr.length > 1) {
                    conditionLabels.add(splitArr[splitArr.length - 1]);
                }
            }
        }
        //排除项
        List<IndicationCondition> notInConditions
                = indicationRepository.getNotInConditions(conditionIndexVO.getConceptName(), conditionIndexVO.getConceptLabel());
        List<Long> notInIds = Lists.newArrayList();
        if (ListUtil.isNotEmpty(notInConditions) && conditionIndexVO.getRuleType().equals("文本类型")) {
            notInIds = notInConditions.stream().map(i -> i.getId()).distinct().collect(Collectors.toList());
        }
        List<IndicationCondition> conditions = indicationRepository.conditionIndex(conditionLabels,
                notInIds,
                conditionIndexVO.getConditionName(),
                conditionIndexVO.getConditionName(),
                100);

        return conditions;
    }
}